'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var turf = require('@turf/turf');
var fp = require('lodash/fp');
var mathjs = require('mathjs');

/**
*    Returns the exact "air" distance in metres between two WGS84 locations.
*    Does not consider the altitude
    http://www.movable-type.co.uk/scripts/latlong.html
*/

function getDistanceMetres(location1, location2) {
  const R = 6378137.0; // Radius of "spherical" earth

  const lat1 = location1[0] * Math.PI / 180;
  const lon1 = location1[1] * Math.PI / 180;
  const lat2 = location2[0] * Math.PI / 180;
  const lon2 = location2[1] * Math.PI / 180;
  const dlat = lat2 - lat1;
  const dlon = lon2 - lon1;
  const a = Math.sin(dlat / 2) * Math.sin(dlat / 2) + Math.cos(lat1) * Math.cos(lat2) * Math.sin(dlon / 2) * Math.sin(dlon / 2);
  const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
  const d = R * c;
  return d;
}
/**
*    Returns the exact "air" distance in metres between two WGS84 locations.
*   Consider the altitude
    http://www.movable-type.co.uk/scripts/latlong.html
*/

function getDistanceMetres3D(location1, location2) {
  const twoDimensionalDistance = getDistanceMetres(convertLocationDictToSterblueList(location1), convertLocationDictToSterblueList(location2));
  return Math.sqrt(Math.pow(twoDimensionalDistance, 2) + Math.pow(location1.altitude - location2.altitude, 2));
}
function getDistanceMetresLTP3D(location1, location2) {
  return mathjs.norm(fp.map(index => location2[index] - location1[index], fp.range(0, location1.length)));
}

/**
 * @module utils/convert/waypoints
 */
/**
 *    Returns the exact "air" distance in metres between two WGS84 locations.
 *    Does not consider the altitude
 *    http://www.movable-type.co.uk/scripts/latlong.html
 *    Inputs and output are dict
 */

function getDistanceMetresDict(location1, location2) {
  return getDistanceMetres(convertLocationDictToSterblueList(location1), convertLocationDictToSterblueList(location2));
}
function convertLocationDictToSterblueList(dict) {
  const sterblueList = [dict.latitude, dict.longitude, dict.altitude];
  return sterblueList;
}
function convertLocationDictToGeojsonList(dict) {
  const geoJsonList = [dict.longitude, dict.latitude, dict.altitude];
  return geoJsonList;
}
function convertLocationDictToPoint(dict) {
  const geoJsonPoint = turf.point([dict.longitude, dict.latitude, dict.altitude]);
  return geoJsonPoint;
}
function convertPointToDict(point) {
  const dict = {
    latitude: point.geometry.coordinates[1],
    longitude: point.geometry.coordinates[0],
    altitude: point.geometry.coordinates[2]
  };
  return dict;
}
function convertPointToGeoJsonList(point) {
  const geoJsonList = point.geometry.coordinates;
  return geoJsonList;
}
function convertPointToSterblueList(point) {
  const sterblueList = [point.geometry.coordinates[1], point.geometry.coordinates[0], point.geometry.coordinates[2]];
  return sterblueList;
}
function convertSterblueListToDict(sterblueList) {
  const dict = {
    latitude: sterblueList[0],
    longitude: sterblueList[1],
    altitude: sterblueList[2]
  };
  return dict;
}
function convertSterblueListToGeoJsonList(sterblueList) {
  const geoJsonList = [sterblueList[1], sterblueList[0], sterblueList[2]];
  return geoJsonList;
}
function convertSterblueListToPoint(sterblueList) {
  const geoJsonPoint = turf.point([sterblueList[1], sterblueList[0], sterblueList[2]]);
  return geoJsonPoint;
}
function convertGeoJsonListToPoint(geoJsonList) {
  const geoJsonPoint = turf.point(geoJsonList);
  return geoJsonPoint;
}
function convertGeoJsonListToDict(geoJsonList) {
  const dict = {
    latitude: geoJsonList[1],
    longitude: geoJsonList[0],
    altitude: geoJsonList[2]
  };
  return dict;
}
function convertGeoJsonListToSterblueList(geoJsonList) {
  const sterblueList = [geoJsonList[1], geoJsonList[0], geoJsonList[2]];
  return sterblueList;
}

exports.convertGeoJsonListToDict = convertGeoJsonListToDict;
exports.convertGeoJsonListToPoint = convertGeoJsonListToPoint;
exports.convertGeoJsonListToSterblueList = convertGeoJsonListToSterblueList;
exports.convertLocationDictToGeojsonList = convertLocationDictToGeojsonList;
exports.convertLocationDictToPoint = convertLocationDictToPoint;
exports.convertLocationDictToSterblueList = convertLocationDictToSterblueList;
exports.convertPointToDict = convertPointToDict;
exports.convertPointToGeoJsonList = convertPointToGeoJsonList;
exports.convertPointToSterblueList = convertPointToSterblueList;
exports.convertSterblueListToDict = convertSterblueListToDict;
exports.convertSterblueListToGeoJsonList = convertSterblueListToGeoJsonList;
exports.convertSterblueListToPoint = convertSterblueListToPoint;
exports.getDistanceMetres = getDistanceMetres;
exports.getDistanceMetres3D = getDistanceMetres3D;
exports.getDistanceMetresDict = getDistanceMetresDict;
exports.getDistanceMetresLTP3D = getDistanceMetresLTP3D;
